{% macro rnafusion_scout_files(scout_files, case_id, case_name) %}
  <table class="table w-auto align-middle" aria-describedby="rnafusion-scout-table">
    <tbody>
      {% if scout_files.vcf_fusion != "N/A" %}
        <tr>
          <th scope="row">Kliniska Fusionsvarianter</th>
          <td>{{ scout_files.vcf_fusion.replace(case_id, case_name) }}</td>
        </tr>
      {% endif %}
    </tbody>
  </table>
{% endmacro %}
